// MainWindow.cs
//
//  Copyright (C) 2008 Paul Lange
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//


using System;
using Gtk;


namespace Sterling
{	
	public partial class MainWindow : Gtk.Window
	{
		TreeModelFilter _filter;
		ListStore _store;
		
		int _filterStatus;
		
		public MainWindow() : base(Gtk.WindowType.Toplevel)
		{
			this.Build();
			this.DeleteEvent += delegate { Application.Quit(); };
			
			_filterStatus = 0;
			
			initializeGUI();
			
			loadData();
		}
		
#region UI stuff
		private void initializeGUI()
		{
			//TODO better column sizing
			TreeViewColumn dateColumn = new TreeViewColumn ();
			dateColumn.Title = "Date";
			dateColumn.Sizing = TreeViewColumnSizing.Autosize;			
			
			TreeViewColumn descColumn = new TreeViewColumn ();
			descColumn.Title = "Description";
			descColumn.Sizing = TreeViewColumnSizing.GrowOnly;
			descColumn.MinWidth = 200;
			
			TreeViewColumn amountColumn = new TreeViewColumn ();
			amountColumn.Title = "Amount";
			amountColumn.Sizing = TreeViewColumnSizing.Autosize;
			
			TreeViewColumn balanceColumn = new TreeViewColumn();
			balanceColumn.Title = "Balance";
			balanceColumn.Sizing = TreeViewColumnSizing.Autosize;
			
			CellRendererText dateRenderer = new CellRendererText();			
			dateColumn.PackStart(dateRenderer, false);
			dateColumn.AddAttribute(dateRenderer, "text", 0);
			
			CellRendererText descRenderer = new CellRendererText();	
			descColumn.PackStart(descRenderer, false);
			descColumn.AddAttribute(descRenderer, "text", 1);
			
			CellRendererText amountRenderer = new CellRendererText();	
			amountColumn.PackStart(amountRenderer, false);
			amountColumn.AddAttribute(amountRenderer, "text", 2);
			
			CellRendererText balanceRenderer = new CellRendererText();	
			balanceColumn.PackStart(balanceRenderer, false);
			balanceColumn.AddAttribute(balanceRenderer, "text", 3);
			
			dateColumn.SetCellDataFunc(dateRenderer, new TreeCellDataFunc(renderDate));
			descColumn.SetCellDataFunc(descRenderer, new TreeCellDataFunc(renderDescription));
			amountColumn.SetCellDataFunc(amountRenderer, new TreeCellDataFunc(renderAmount));
			balanceColumn.SetCellDataFunc(balanceRenderer, new TreeCellDataFunc(renderBalance));

			dataView.AppendColumn(dateColumn);
			dataView.AppendColumn(descColumn);
			dataView.AppendColumn(amountColumn);
			dataView.AppendColumn(balanceColumn);
			
			_store = new ListStore(typeof(Transaction), typeof(double));
			
			_filter = new TreeModelFilter(_store, null);
			_filter.VisibleFunc = new TreeModelFilterVisibleFunc(searchAndFilter);
			
			dataView.Model = _filter;
		}
		
		private void renderAmount (TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter)
		{
			Transaction trans = (Transaction) model.GetValue (iter, 0);
	 
			(cell as Gtk.CellRendererText).Weight = 10000000;
			(cell as Gtk.CellRendererText).Text = string.Format("{0:0.00}", trans.Amount);
	 
			if (trans.Amount < 0)
			{
				(cell as Gtk.CellRendererText).Foreground = "red";
			} else 
			{
				if (trans.Amount > 0)
					(cell as Gtk.CellRendererText).Text = "+" + (cell as Gtk.CellRendererText).Text;
				
				(cell as Gtk.CellRendererText).Foreground = "darkgreen";
			}
		}
	 
		private void renderDate (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
		{
			Transaction trans = (Transaction) model.GetValue (iter, 0);
			(cell as Gtk.CellRendererText).Text = trans.Date.ToShortDateString();
		}
		
		private void renderDescription (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
		{
			Transaction trans = (Transaction) model.GetValue (iter, 0);
			(cell as Gtk.CellRendererText).Text = trans.Description;
			(cell as Gtk.CellRendererText).Alignment = Pango.Alignment.Center;
		}
		
		private void renderBalance (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
		{
			//TODO: implement this
			double balance = (double) model.GetValue (iter, 1);
			
			string plus = (balance > 0) ? "+" : "";
			
			(cell as Gtk.CellRendererText).Weight = 10000000;
			(cell as Gtk.CellRendererText).Alignment = Pango.Alignment.Right;
			(cell as Gtk.CellRendererText).Text = plus + string.Format("{0:0.00}", balance);
		}
#endregion
		
		private void loadData()
		{
			_store.AppendValues (new Transaction(0d, "Start balancing", DateTime.Now), 0d);
			
			_store.AppendValues (new Transaction(-1.20, "test", DateTime.Now),-1.2);
			_store.AppendValues (new Transaction(1.23, "test", DateTime.Now),0.23);
		}
				
		[GLib.ConnectBefore]
		protected virtual void OnDataViewButtonPressEvent (object o, Gtk.ButtonPressEventArgs args)
		{
			if (args.Event.Type == Gdk.EventType.ButtonPress && args.Event.Button == 3) 
			{
				Menu context = new Menu();
				ImageMenuItem edit = new ImageMenuItem(Stock.Edit, null);
				edit.Activated += new EventHandler(ContextMenUEditActivated);
				
				context.Append(edit);
				
				System.Console.WriteLine("cooler");
				
				PopupContextMenuAtLocation(context, (int)args.Event.X, (int)args.Event.Y);
			}
		}
		
		void PopupContextMenuAtLocation (Gtk.Menu menu, int x, int y)
		{
			menu.ShowAll ();
			Gtk.MenuPositionFunc pos_menu_func = null;

			// Set up the funtion to position the context menu
			// if we were called by the keyboard Gdk.Key.Menu.
			if (x == 0 && y == 0)
			    pos_menu_func = PositionContextMenu;

			menu.Popup (null, null,	pos_menu_func, 0, Gtk.Global.CurrentEventTime);
		}
		
		// This is needed for when the user opens
		// the context menu with the keyboard.
		void PositionContextMenu (Gtk.Menu menu, out int x, out int y, out bool push_in)
		{
			Gtk.TreeIter iter;
			Gtk.TreePath path;
			Gtk.TreeSelection selection;

			// Set default "return" values
			push_in = false; // not used
			x = 0;
			y = 0;

			selection = dataView.Selection;
			if (!selection.GetSelected (out iter))
			    return;

			path = _filter.GetPath (iter);

			int pos_x = 0;
			int pos_y = 0;

			GetWidgetScreenPos (dataView, ref pos_x, ref pos_y);
			Gdk.Rectangle cell_rect = dataView.GetCellArea (path, dataView.Columns [0]);

			// Add 100 to x so it's not be at the far left
			x = pos_x + cell_rect.X + 100;
			y = pos_y + cell_rect.Y;
		}

		// Walk the widget hiearchy to figure out
		// where to position the context menu.
		void GetWidgetScreenPos (Gtk.Widget widget, ref int x, ref int y)
		{
			int widget_x;
			int widget_y;

			if (widget is Gtk.Window) 
			{
				((Gtk.Window) widget).GetPosition (out widget_x, out widget_y);
			} else 
			{
				GetWidgetScreenPos (widget.Parent, ref x, ref y);

				// Special case the TreeView because it adds
				// too much since it's in a scrolled window.
				if (widget == dataView) 
				{
				    widget_x = 2;
				    widget_y = 2;
				} else 
				{
				    Gdk.Rectangle widget_rect = widget.Allocation;
				    widget_x = widget_rect.X;
				    widget_y = widget_rect.Y;
				}
			}

			x += widget_x;
			y += widget_y;
		}
		
		void ContextMenUEditActivated(object o, EventArgs args)
		{
			TreeIter iter;
			
			if (dataView.Selection.GetSelected(out iter))
			{
				Transaction item = (Transaction)_filter.GetValue(iter, 0);
				
				TransactionDialog dialog = new TransactionDialog(item);
				
				//TODO: Handle editing
				
				if (dialog.Run() == (int)(ResponseType.Ok))
				{
					Transaction t = dialog.Current;
					//TODO: also store in Account or implement account usen liststore
					_store.AppendValues(t);
				}
				
				dialog.Destroy();
			}			
		}
		
		private void showNewTransactionDialog()
		{
			TransactionDialog dialog = new TransactionDialog();
			
			if (dialog.Run() == (int)(ResponseType.Ok))
			{
				Transaction t = dialog.Current;
				//TODO: also store in Account or implement account usen liststore
				_store.AppendValues(t);
			}
			
			dialog.Destroy();
		}

		protected virtual void OnInfoActionActivated (object sender, System.EventArgs e)
		{
			AboutDialog dialog = new AboutDialog();
			
			dialog.ProgramName = "Sterling Accounting";
			dialog.Comments = "A easy transaction tracking application.";
			dialog.Version = "0.1";
			dialog.Authors = new string[] {"Paul Lange"};
			
			dialog.Icon = Gdk.Pixbuf.LoadFromResource("hicolor_apps_16x16_sterling.png");
			
			dialog.Run();
			dialog.Destroy();				
		}

		protected virtual void OnNewTransactionActivated (object sender, EventArgs e)
		{
			//TODO: handle editing
			showNewTransactionDialog();
		}

		protected virtual void OnQuitActionActivated (object sender, System.EventArgs e)
		{
			Application.Quit();
		}

#region Searching and Filtering
		protected virtual void OnFilterComboChanged (object sender, System.EventArgs e)
		{
			ComboBox filter = sender as ComboBox;
			
			if (filter==null)
				return;
			
			TreeIter iter;
			
			if (filter.GetActiveIter(out iter))
			{				
				string key = (string) filter.Model.GetPath(iter).ToString();
				
				int filterKey = 0;
				if (int.TryParse(key, out filterKey))
				{
					_filterStatus = filterKey;
					_filter.Refilter();
				}
			}
		}

		protected virtual void OnSearchEntryChanged (object sender, System.EventArgs e)
		{
			_filter.Refilter();
		}	
		
		private bool searchAndFilter(TreeModel model, TreeIter iter)
		{
			Transaction trans = model.GetValue(iter, 0) as Transaction;
			
			if (trans == null)
				return false;
			
			return (filterItems(trans) && searchItems(trans));
		}
		
		private bool filterItems(Transaction trans)
		{
			switch (_filterStatus)
			{
				case 0:
					return true;
				case 1:
					if (trans.Amount >= 0)
						return true;
					else
						return false;
				case 2:
					if (trans.Amount < 0)
						return true;
					else
						return false;
				default:
					return true;
			}
		}
		
		private bool searchItems(Transaction trans)
		{			
			string searchText = searchEntry.Text;
			
			if (searchText == "")
				return true;
			
			if (trans.Date.ToShortDateString().Contains(searchText))
				return true;
			
			if (trans.Amount.ToString().Contains(searchText))
				return true;			
			
			return trans.Description.Contains(searchText);
		}
#endregion
	}
}
